package cat.hack3.codingtests.marsrover.ui.console;

import cat.hack3.codingtests.marsrover.api.RotableRiderRover;
import cat.hack3.codingtests.marsrover.api.cartography.Coordinates;
import cat.hack3.codingtests.marsrover.api.cartography.Direction;
import cat.hack3.codingtests.marsrover.api.cartography.PlanetMap;

import java.util.InputMismatchException;
import java.util.List;
import java.util.Scanner;

import static cat.hack3.codingtests.marsrover.ui.console.UICommons.output;

public class RoverInitializer {
    private final Scanner reader;
    private final DirectionRetriever directionRetriever;


    public RoverInitializer(Scanner reader) {
        this.reader = reader;
        directionRetriever = new DirectionRetriever(reader);
    }

    RotableRiderRover initializeFromUserInputs() {
        output("Please, introduce the latitude  map size:");
        int mapHeight = takeIntegerInput();
        output("Please, introduce the longitude map size:");
        int mapWidth = takeIntegerInput();

        output("Great! Now please introduce the initial location of the Rover.");
        output("Latitude:");
        int latitudeStartingPoint = takeIntegerInput();
        output("Longitude:");
        int longitudeStartingPoint = takeIntegerInput();

        Direction startingDirection = directionRetriever.retrieveDirection();

        RotableRiderRover rover = deployRover(mapHeight, mapWidth, latitudeStartingPoint, longitudeStartingPoint, startingDirection, List.of());

        output(PresentationMessage.READY_MESSAGE);
        reader.next(); //input ignored

        return rover;
    }

    private int takeIntegerInput() {
        try {
            return reader.nextInt();
        } catch (InputMismatchException e) {
            output("(please, introduce a number)");
            reader.next();
            return takeIntegerInput();
        }
    }

    RotableRiderRover autoInitialize() { //to test more quickly
        List<Coordinates> obstacles = List.of(
                Coordinates.of(3, 3),
                Coordinates.of(5, 5),
                Coordinates.of(7, 7),
                Coordinates.of(9, 9),
                Coordinates.of(10, 10)
        );
        return deployRover(10, 10, 2, 3, Direction.SOUTH, obstacles);
    }

    private RotableRiderRover deployRover(int mapHeight, int mapWidth, int latitudeStartingPoint, int longitudeStartingPoint, Direction startingDirection, List<Coordinates> obstaclesLocalizations) {
        var startingCoordinates = Coordinates.of(latitudeStartingPoint, longitudeStartingPoint);

        var mapProvider = UICommons.getImplProviderOf(PlanetMap.Provider.class);
        var roverProvider = UICommons.getImplProviderOf(RotableRiderRover.Provider.class);
        var marsMap = mapProvider.provideWith(mapHeight, mapWidth, startingCoordinates, obstaclesLocalizations);
        return roverProvider.provideWith(marsMap, startingDirection);
    }
}
